# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import importlib

from django.test.signals import setting_changed
from django.contrib.sites.shortcuts import get_current_site
from django.core.urlresolvers import reverse
from django.views.generic import FormView, TemplateView
from django.dispatch import receiver
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .forms import ContactForm
from .helpers import ContactFormHelper
from .models import ContactFormSettings
from .mails import ContactMail


_form_helper_module = uninitialized = object()


@receiver(setting_changed)
def on_setting_changed(sender, setting, **kw):
    global _form_helper_module
    if setting == 'CONTACT_FORM_HELPERS_MODULE':
        _form_helper_module = uninitialized


def get_form_helper_module():
    global _form_helper_module
    if _form_helper_module is uninitialized:
        module_name = getattr(settings, 'CONTACT_FORM_HELPERS_MODULE', None)
        if module_name is not None:
            _form_helper_module = importlib.import_module(module_name)
    return _form_helper_module


class FormHelperMixin(object):
    form_helper_class = None

    def get_form_helper_class(self):
        if self.form_helper_class is None:
            raise ImproperlyConfigured(u'Missing form_helper_class attribute on class {}'.format(self.__class__))

        module = get_form_helper_module()
        try:
            return getattr(module, self.form_helper_class.__name__)
        except AttributeError:
            pass
        return self.form_helper_class

    def get_form_helper(self):
        helper_class = self.get_form_helper_class()
        kwargs = self.get_form_helper_kwargs()
        return helper_class(**kwargs)

    def get_form_helper_kwargs(self):
        return {}

    def get_context_data(self, **kw):
        context = super(FormHelperMixin, self).get_context_data(**kw)
        context.update({
            'form_helper': self.get_form_helper(),
        })
        return context


class ContactFormView(FormHelperMixin, FormView):
    form_class = ContactForm
    form_helper_class = ContactFormHelper
    template_name = 'contact_form/contact_form.html'

    def get_success_url(self):
        return reverse('contact_form_success')

    def form_valid(self, form):
        contact = form.save()
        site = get_current_site(self.request)
        cm = ContactMail(site, contact)

        email = cm.get_email()
        email.send()

        return super(ContactFormView, self).form_valid(form)


class ContactFormSuccessView(TemplateView):
    template_name = "contact_form/contact_form_success.html"

    def get_context_data(self, **kwargs):
        context = super(ContactFormSuccessView, self).get_context_data(**kwargs)
        html_success = ContactFormSettings.on_site.get().success_html
        context["message_success_content"] = html_success
        return context
